"""Top-level package for aws_pcluster_bootstrap_helpers."""

__author__ = """Jillian Rowe"""
__email__ = "jillian@dabbleofdevops.com"
__version__ = "0.1.0"

from . import _version
__version__ = _version.get_versions()['version']
